//! Rhai engine creation and registration for Git module
//!
//! This module provides functions to create and configure a Rhai engine with Git functionality.

use rhai::{Engine, EvalAltResult};
use crate::git::{GitTree, GitRepo};
use super::wrapper::{
    create_git_tree, list_repositories, find_repositories, get_repositories,
    get_repo_path, has_changes, pull_repository, reset_repository,
    commit_changes, push_changes, parse_git_url_wrapper
};

/// Create a Rhai engine with Git functionality
pub fn create_rhai_engine() -> Engine {
    let mut engine = Engine::new();
    
    // Register the Git module
    register_git_module(&mut engine).expect("Failed to register Git module");
    
    engine
}

/// Register Git module functions with the Rhai engine
pub fn register_git_module(engine: &mut Engine) -> Result<(), Box<EvalAltResult>> {
    // Register GitTree constructor
    engine.register_fn("new_git_tree", create_git_tree);
    
    // Register GitTree methods
    engine.register_fn("list_repositories", list_repositories);
    engine.register_fn("find_repositories", find_repositories);
    engine.register_fn("get_repositories", get_repositories);
    
    // Register GitRepo methods
    engine.register_fn("get_repo_path", get_repo_path);
    engine.register_fn("has_changes", has_changes);
    engine.register_fn("pull_repository", pull_repository);
    engine.register_fn("reset_repository", reset_repository);
    engine.register_fn("commit_changes", commit_changes);
    engine.register_fn("push_changes", push_changes);
    
    // Register utility functions
    engine.register_fn("parse_git_url", parse_git_url_wrapper);
    
    // Register types
    engine.register_type_with_name::<GitTree>("GitTree");
    engine.register_type_with_name::<GitRepo>("GitRepo");
    
    Ok(())
}