use rhai::{Dynamic, Engine, EvalAltResult, Module, NativeCallContext, FuncRegistration, FnNamespace, Identifier};
use std::sync::Arc;
use crate::git::{GitTree, GitRepo};
use super::wrapper;

/// Creates and returns a Rhai module with Git functionality.
/// 
/// This module exposes Git-related functions to Rhai scripts.
pub fn create_git_module() -> Module {
    let mut module = Module::new();
    
    // Register GitTree functions
    FuncRegistration::new("create_git_tree")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["base_path: &str", "GitTree"])
        .with_comments(["/// Create a new GitTree with the specified base path",
                       "/// - base_path: Base directory path to search for git repositories",
                       "/// Returns a GitTree instance"])
        .set_into_module(&mut module, wrapper::create_git_tree);
    
    FuncRegistration::new("list_repositories")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["tree: &mut GitTree", "Array"])
        .with_comments(["/// List all git repositories under the base path",
                       "/// - tree: GitTree instance",
                       "/// Returns an array of GitRepo instances"])
        .set_into_module(&mut module, wrapper::list_repositories);
    
    FuncRegistration::new("find_repositories")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["tree: &mut GitTree", "pattern: &str", "Array"])
        .with_comments(["/// Find repositories matching a pattern",
                       "/// - tree: GitTree instance",
                       "/// - pattern: Pattern to match repository names",
                       "/// Returns an array of matching GitRepo instances"])
        .set_into_module(&mut module, wrapper::find_repositories);
    
    FuncRegistration::new("get_repositories")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["tree: &mut GitTree", "path_or_url: &str", "Array"])
        .with_comments(["/// Get repositories based on a path pattern or URL",
                       "/// - tree: GitTree instance",
                       "/// - path_or_url: Path pattern or URL to match",
                       "/// Returns an array of matching GitRepo instances"])
        .set_into_module(&mut module, wrapper::get_repositories);
    
    // Register GitRepo functions
    FuncRegistration::new("get_repo_path")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["repo: &mut GitRepo", "String"])
        .with_comments(["/// Get the path of the repository",
                       "/// - repo: GitRepo instance",
                       "/// Returns the path of the repository as a string"])
        .set_into_module(&mut module, wrapper::get_repo_path);
    
    FuncRegistration::new("has_changes")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["repo: &mut GitRepo", "bool"])
        .with_comments(["/// Check if the repository has uncommitted changes",
                       "/// - repo: GitRepo instance",
                       "/// Returns true if there are uncommitted changes, false otherwise"])
        .set_into_module(&mut module, wrapper::has_changes);
    
    FuncRegistration::new("pull_repository")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["repo: &mut GitRepo", "String"])
        .with_comments(["/// Pull the latest changes from the remote repository",
                       "/// - repo: GitRepo instance",
                       "/// Returns a success message or throws an error"])
        .set_into_module(&mut module, wrapper::pull_repository);
    
    FuncRegistration::new("reset_repository")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["repo: &mut GitRepo", "String"])
        .with_comments(["/// Reset any local changes in the repository",
                       "/// - repo: GitRepo instance",
                       "/// Returns a success message or throws an error"])
        .set_into_module(&mut module, wrapper::reset_repository);
    
    FuncRegistration::new("commit_changes")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["repo: &mut GitRepo", "message: &str", "String"])
        .with_comments(["/// Commit changes in the repository",
                       "/// - repo: GitRepo instance",
                       "/// - message: Commit message",
                       "/// Returns a success message or throws an error"])
        .set_into_module(&mut module, wrapper::commit_changes);
    
    FuncRegistration::new("push_changes")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["repo: &mut GitRepo", "String"])
        .with_comments(["/// Push changes to the remote repository",
                       "/// - repo: GitRepo instance",
                       "/// Returns a success message or throws an error"])
        .set_into_module(&mut module, wrapper::push_changes);
    
    FuncRegistration::new("parse_git_url")
        .with_namespace(FnNamespace::Global)
        .with_params_info(["url: &str", "Array"])
        .with_comments(["/// Parse a git URL to extract the server, account, and repository name",
                       "/// - url: Git URL to parse",
                       "/// Returns an array with [server, account, repo]"])
        .set_into_module(&mut module, wrapper::parse_git_url_wrapper);
    
    module
}
