//! Rhai wrappers for Git module functions
//!
//! This module provides Rhai wrappers for the functions in the Git module.

use rhai::{EvalAltResult, Array, Dynamic, Position};
use crate::git::{GitTree, GitRepo, GitError, parse_git_url};
use super::generic_wrapper::{RhaiWrapper, ToRhai};

// Helper function to convert GitError to EvalAltResult
fn git_error_to_rhai_error<T>(result: Result<T, GitError>) -> Result<T, Box<EvalAltResult>> {
    result.map_err(|e| {
        Box::new(EvalAltResult::ErrorRuntime(
            format!("Git error: {}", e).into(),
            Position::NONE
        ))
    })
}

//
// GitTree Functions
//

/// Create a new GitTree with the specified base path
pub fn create_git_tree(base_path: &str) -> Result<GitTree, Box<EvalAltResult>> {
    git_error_to_rhai_error(GitTree::new(base_path))
}

/// List all git repositories under the base path
pub fn list_repositories(tree: &mut GitTree) -> Result<Array, Box<EvalAltResult>> {
    git_error_to_rhai_error(tree.list()).map(|repos| {
        let array: Array = repos.into_iter()
            .map(|repo| Dynamic::from(repo))
            .collect();
        array
    })
}

/// Find repositories matching a pattern
pub fn find_repositories(tree: &mut GitTree, pattern: &str) -> Result<Array, Box<EvalAltResult>> {
    git_error_to_rhai_error(tree.find(pattern)).map(|repos| {
        let array: Array = repos.into_iter()
            .map(|repo| Dynamic::from(repo))
            .collect();
        array
    })
}

/// Get repositories based on a path pattern or URL
pub fn get_repositories(tree: &mut GitTree, path_or_url: &str) -> Result<Array, Box<EvalAltResult>> {
    git_error_to_rhai_error(tree.get(path_or_url)).map(|repos| {
        let array: Array = repos.into_iter()
            .map(Dynamic::from)
            .collect();
        array
    })
}

//
// GitRepo Functions
//

/// Get the path of the repository
pub fn get_repo_path(repo: &mut GitRepo) -> String {
    repo.path().to_string()
}

/// Check if the repository has uncommitted changes
pub fn has_changes(repo: &mut GitRepo) -> Result<bool, Box<EvalAltResult>> {
    git_error_to_rhai_error(repo.has_changes())
}

/// Pull the latest changes from the remote repository
pub fn pull_repository(repo: &mut GitRepo) -> Result<String, Box<EvalAltResult>> {
    match repo.pull() {
        Ok(_) => Ok(String::from("Successfully pulled changes")),
        Err(err) => Err(Box::new(EvalAltResult::ErrorRuntime(
            format!("Error pulling changes: {}", err).into(),
            Position::NONE
        )))
    }
}

/// Reset any local changes in the repository
pub fn reset_repository(repo: &mut GitRepo) -> Result<String, Box<EvalAltResult>> {
    match repo.reset() {
        Ok(_) => Ok(String::from("Successfully reset repository")),
        Err(err) => Err(Box::new(EvalAltResult::ErrorRuntime(
            format!("Error resetting repository: {}", err).into(),
            Position::NONE
        )))
    }
}

/// Commit changes in the repository
pub fn commit_changes(repo: &mut GitRepo, message: &str) -> Result<String, Box<EvalAltResult>> {
    match repo.commit(message) {
        Ok(_) => Ok(String::from("Successfully committed changes")),
        Err(err) => Err(Box::new(EvalAltResult::ErrorRuntime(
            format!("Error committing changes: {}", err).into(),
            Position::NONE
        )))
    }
}

/// Push changes to the remote repository
pub fn push_changes(repo: &mut GitRepo) -> Result<String, Box<EvalAltResult>> {
    match repo.push() {
        Ok(_) => Ok(String::from("Successfully pushed changes")),
        Err(err) => Err(Box::new(EvalAltResult::ErrorRuntime(
            format!("Error pushing changes: {}", err).into(),
            Position::NONE
        )))
    }
}

/// Parse a git URL to extract the server, account, and repository name
pub fn parse_git_url_wrapper(url: &str) -> Array {
    let (server, account, repo) = parse_git_url(url);
    let mut array = Array::new();
    array.push(Dynamic::from(server));
    array.push(Dynamic::from(account));
    array.push(Dynamic::from(repo));
    array
}

// Implementation of ToRhai for GitTree
impl ToRhai for GitTree {
    fn to_rhai(&self) -> Dynamic {
        Dynamic::from(self.clone())
    }
}

// Implementation of ToRhai for GitRepo
impl ToRhai for GitRepo {
    fn to_rhai(&self) -> Dynamic {
        Dynamic::from(self.clone())
    }
}

// Implementation of ToRhai for String
impl ToRhai for String {
    fn to_rhai(&self) -> Dynamic {
        Dynamic::from(self.clone())
    }
}

// Implementation of ToRhai for bool
impl ToRhai for bool {
    fn to_rhai(&self) -> Dynamic {
        Dynamic::from(*self)
    }
}

// Implementation of ToRhai for Vec<String>
impl ToRhai for Vec<String> {
    fn to_rhai(&self) -> Dynamic {
        let array: Array = self.iter()
            .map(|s| Dynamic::from(s.clone()))
            .collect();
        Dynamic::from(array)
    }
}

// Implementation of ToRhai for Option<GitRepo>
impl<T: ToRhai> ToRhai for Option<T> {
    fn to_rhai(&self) -> Dynamic {
        match self {
            Some(value) => value.to_rhai(),
            None => Dynamic::UNIT
        }
    }
}